@@ -19,6 +19,7 @@
 
 import string
 import itertools
+import selinux
 
 # OVERVIEW
 #
@@ -122,6 +123,12 @@ class Node(PolicyBase):
     def roles(self):
         return itertools.ifilter(lambda x: isinstance(x, Role), walktree(self))
 
+    def role_allows(self):
+        return itertools.ifilter(lambda x: isinstance(x, RoleAllow), walktree(self))
+
+    def role_types(self):
+        return itertools.ifilter(lambda x: isinstance(x, RoleType), walktree(self))
+
     def __str__(self):
         if self.comment:
             return str(self.comment) + "\n" + self.to_string()
@@ -259,7 +266,7 @@ class SecurityContext(Leaf):
         self.user = ""
         self.role = ""
         self.type = ""
-        self.level = ""
+        self.level = None
         if context is not None:
             self.from_string(context)
 
@@ -282,7 +289,7 @@ class SecurityContext(Leaf):
             # FUTURE - normalize level fields to allow more comparisons to succeed.
             self.level = string.join(fields[3:], ':')
         else:
-            self.level = ""
+            self.level = None
 
     def __eq__(self, other):
         """Compare two SecurityContext objects - all fields must be exactly the
@@ -295,7 +302,7 @@ class SecurityContext(Leaf):
                self.type == other.type and \
                self.level == other.level
 
-    def to_string(self, default_level="s0"):
+    def to_string(self, default_level=None):
         """Return a string representing this security context.
 
         By default, the string will contiain a MCS / MLS level
@@ -311,8 +318,11 @@ class SecurityContext(Leaf):
               'user:role:type:level'.
         """
         fields = [self.user, self.role, self.type]
-        if self.level == "":
-            if default_level != "":
+        if self.level is None:
+            if default_level is None:
+                if selinux.is_selinux_mls_enabled() == 1:
+                    fields.append("s0")
+            else:
                 fields.append(default_level)
         else:
             fields.append(self.level)
@@ -494,6 +504,15 @@ class RoleAllow(Leaf):
         return "allow %s %s;" % (self.src_roles.to_comma_str(),
                                  self.tgt_roles.to_comma_str())
 
+class RoleType(Leaf):
+    def __init__(self, parent=None):
+        Leaf.__init__(self, parent)
+        self.role = ""
+        self.types = IdSet()
+
+    def to_string(self):
+        return "role %s types %s;" % (self.role, self.types.to_comma_str())
+
 class ModuleDeclaration(Leaf):
     def __init__(self, parent=None):
         Leaf.__init__(self, parent)
@@ -599,6 +618,41 @@ class NetifCon(Leaf):
     def to_string(self):
         return "netifcon %s %s %s" % (self.interface, str(self.interface_context),
                                    str(self.packet_context))
+class PirqCon(Leaf):
+    def __init__(self, parent=None):
+        Leaf.__init__(self, parent)
+        self.pirq_number = ""
+        self.context = None
+
+    def to_string(self):
+        return "pirqcon %s %s" % (self.pirq_number, str(self.context))
+
+class IomemCon(Leaf):
+    def __init__(self, parent=None):
+        Leaf.__init__(self, parent)
+        self.device_mem = ""
+        self.context = None
+
+    def to_string(self):
+        return "iomemcon %s %s" % (self.device_mem, str(self.context))
+
+class IoportCon(Leaf):
+    def __init__(self, parent=None):
+        Leaf.__init__(self, parent)
+        self.ioport = ""
+        self.context = None
+
+    def to_string(self):
+        return "ioportcon %s %s" % (self.ioport, str(self.context))
+
+class PciDeviceCon(Leaf):
+    def __init__(self, parent=None):
+        Leaf.__init__(self, parent)
+        self.device = ""
+        self.context = None
+
+    def to_string(self):
+        return "pcidevicecon %s %s" % (self.device, str(self.context))
 
 # Reference policy specific types
 
